#ifndef MSP_INPUT_GESTUREDETECTOR_H_
#define MSP_INPUT_GESTUREDETECTOR_H_

#include "device.h"

namespace Msp {
namespace Input {

class Touchscreen;

enum Gesture
{
	GESTURE_NONE,
	GESTURE_TAP,
	GESTURE_TAP_2,
	GESTURE_DRAG,
	GESTURE_DRAG_2,
	GESTURE_PINCH,
	GESTURE_ROTATE
};

/**
Interprets events from a Touchscreen as high-level gestures.  One button is
provided for each type of gesture.  Axes 0 and 1 indicate the starting position
of the gesture; axes 2 and 3 track its progress.  The progress axis may exhibit
absolute values greater than one.
*/
class GestureDetector: public Device
{
private:
	enum
	{
		MAX_POINTS = 3
	};

	struct TouchPoint
	{
		bool down;
		float down_x;
		float down_y;
		float x;
		float y;
		bool threshold_exceeded;

		TouchPoint();
	};

	Touchscreen &touchscreen;
	TouchPoint points[MAX_POINTS];
	Gesture current_gesture;
	unsigned active_points;
	Gesture pending_tap;
	bool invalid_gesture;
	float threshold_x_sq;
	float threshold_y_sq;

public:
	GestureDetector(Touchscreen &);

	virtual std::string get_button_name(unsigned) const;
	virtual std::string get_axis_name(unsigned) const;

private:
	void touch_down(unsigned);
	void touch_up(unsigned);
	void touch_move(unsigned, float, float);
	void start_gesture();
	void set_gesture_location(unsigned);
	void update_progress();
	void end_gesture();
	void window_resized(unsigned, unsigned);
};

} // namespace Input
} // namespace Msp

#endif
